<?php

/*
 * This file is part of Composer.
 *
 * (c) Nils Adermann <naderman@naderman.de>
 *     Jordi Boggiano <j.boggiano@seld.be>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Composer\Autoload;

use Composer\Installer\InstallationManager;
use Composer\Json\JsonFile;
use Composer\Package\Loader\JsonLoader;
use Composer\Package\PackageInterface;
use Composer\Repository\RepositoryInterface;

/**
 * @author Igor Wiedler <igor@wiedler.ch>
 * @author Jordi Boggiano <j.boggiano@seld.be>
 */
class AutoloadGenerator
{
    private $localRepo;
    private $package;
    private $installationManager;

    public function __construct(RepositoryInterface $localRepo, PackageInterface $package, InstallationManager $installationManager)
    {
        $this->localRepo = $localRepo;
        $this->package = $package;
        $this->installationManager = $installationManager;
    }

    public function dump($targetFilename)
    {
        $autoloads = $this->parseAutoloads();

        $file = file_get_contents(__DIR__.'/ClassLoader.php');

        $file .= <<<'EOF'

// autoload.php generated by composer

$loader = new ClassLoader();

EOF;

        if (isset($autoloads['psr-0'])) {
            foreach ($autoloads['psr-0'] as $def) {
                foreach ($def['mapping'] as $prefix => $path) {
                    $exportedPrefix = var_export($prefix, true);
                    $exportedPath = var_export(($def['path'] ? '/'.$def['path'] : '').'/'.$path, true);
                    $file .= <<<EOF
\$loader->add($exportedPrefix, dirname(__DIR__).$exportedPath);

EOF;
                }
            }
        }

        $file .= <<<'EOF'
$loader->register();

EOF;

        file_put_contents($targetFilename, $file);
    }

    private function parseAutoloads()
    {
        $installPaths = array();
        foreach ($this->localRepo->getPackages() as $package) {
            $installPaths[] = array(
                $package,
                $this->installationManager->getInstallPath($package)
            );
        }
        $installPaths[] = array($this->package, '');

        $autoloads = array();
        foreach ($installPaths as $item) {
            list($package, $installPath) = $item;

            if (null !== $package->getTargetDir()) {
                $installPath = substr($installPath, 0, -strlen('/'.$package->getTargetDir()));
            }

            foreach ($package->getAutoload() as $type => $mapping) {
                $autoloads[$type][] = array(
                    'mapping'   => $mapping,
                    'path'      => $installPath,
                );
            }
        }

        return $autoloads;
    }
}
